//  SPDX-FileCopyrightText: 2023 Yury Gubich <blue@macaw.me>
//  SPDX-License-Identifier: GPL-3.0-or-later

#include "register.h"

#include "server/server.h"

Handler::Register::Register(Server* server):
    Handler("register", Request::Method::post),
    server(server)
{}

void Handler::Register::handle(Request& request) {
    std::map form = request.getForm();
    std::map<std::string, std::string>::const_iterator itr = form.find("login");
    if (itr == form.end())
        return error(request, Result::noLogin);

    const std::string& login = itr->second;
    if (login.empty())
        return error(request, Result::emptyLogin);

    //TODO login policies checkup

    itr = form.find("password");
    if (itr == form.end())
        return error(request, Result::noPassword);

    const std::string& password = itr->second;
    if (password.empty())
        return error(request, Result::emptyPassword);

    //TODO password policies checkup

    try {
        server->registerAccount(login, password);
    } catch (const std::exception& e) {
        std::cerr << "Exception on registration:\n\t" << e.what() << std::endl;
        return error(request, Result::unknownError);
    } catch (...) {
        std::cerr << "Unknown exception on registration" << std::endl;
        return error(request, Result::unknownError);
    }

    Response res(request);
    nlohmann::json body = nlohmann::json::object();
    body["result"] = Result::success;

    res.setBody(body);
    res.send();
}

void Handler::Register::error(Request& request, Result result) {
    Response res(request);
    nlohmann::json body = nlohmann::json::object();
    body["result"] = result;

    res.setBody(body);
    res.send();
}
